/*****************************************************************************
 * Copyright 2013 celum Slovakia s r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package com.celum.dbtool.installer;

import com.celum.nyx.DataAccessException;
import com.celum.nyx.interceptor.SqlInterceptor;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.Map;

/**
 * This inteceptor impl. is responsible for templating. Means placeholders in SQL will be replaced
 * by real values and modified SQL is then moved to decorated strategy where is
 * processed.
 *
 * example:
 * <code>
 *     Map<String, Object> values = ...;
 *     values.put("user", "mickeymouse");
 *     ...
 *     VelocityDecorator<ListParser> strategy = VelocityDecorator.decorate(new ListParser(), values);
 *
 *     List<String> sql =
 *         Sql
 *             .loadScriptFromStream(is)
 *             .interceptingSqlWith(new VelocityInterceptor(values))
 *             .viaStrategy(new ListParser())
 *             .returnListOfQueries();
 * </code>
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)

 */
public class VelocityInterceptor implements SqlInterceptor
{
    /** engine does the templating */
    private VelocityEngine engine;

    /** context holds the real values */
    private VelocityContext context;


    /**
     * Constructor
     */
    public VelocityInterceptor(Map<String, ?> values)
    {
        initWith(values);
    }


    /**
     * initialize Velocity engine
     * @param values
     */
    private void initWith(Map<String, ?> values)
    {
        try {
            //initialize velocity
            engine = new VelocityEngine();
            engine.init();

            //create context
            context = new VelocityContext();
            for (Map.Entry<String, ?> e : values.entrySet()) {
                context.put(e.getKey(), e.getValue());
            }
        } catch (Exception e) {
            throw new DataAccessException("Error in velocity interceptor:" + e.getMessage(), e);
        }
    }


    /**
     * {@inheritDoc}
     */
    @Override
    public String preHandleSql(String sql)
    {
        return replacePlaceholdersIn(sql);
    }


    /**
     * {@inheritDoc}
     */
    @Override
    public void afterCompleteSql(String sql)
    {
        //do nothing
    }


    /**
     * method replace velocity placeholders with real
     * values.
     *
     * @param sql original SQL with placeholders
     * @return sql with real values
     */
    private String replacePlaceholdersIn(String sql)
    {
        try {
            StringWriter sw = new StringWriter();
            engine.evaluate(context, sw, "sql", new StringReader(sql));
            sw.flush();
            return sw.toString();
        } catch (IOException e) {
            throw new DataAccessException("Error in velocity decoration:" + e.getMessage());
        }
    }
}

